#!/usr/bin/env python3
# coding: utf-8

# $Id: sge_job_script_degrade.py 2876 2020-04-16 09:41:39Z ltaulell $
# SPDX-License-Identifier: BSD-2-Clause

""" submit a bunch of OpenMP jobs, with parametric files, to SGE """

__author__ = 'ltaulell'
__credits__ = 'Pratyush Pranav <pratyuze AT gmail.com>'

import os
import sys
import time

# computation parameters
mask = [0.9] # mask cutoff value
degrade = [1024,2048] #list of degraded resolutions to analyze, equivalent to Nside of Healpix
fwhm = [10,5] # corresponding fwhm at each resolution..list length must match the length of degrade list, and fwhm supplied in the same order as degrade
#iter=[714]
BEGIN = 200
END = 800 # not included, python3 range is [begin, end[

# inputs & outputs
email = 'pratyuze@gmail.com'
# full path or './' to binary:
binary1 = '~/topos2/helper_func/fits_RDS_mask_RDS_nrm'
binary2 = '/Xnfs/arthuscmb/pranav/codes/filtration_builder_healpix/headerless_streaming_filtration_builder_healpix_polyhedron_superlevelset'
binary3 = '/Xnfs/arthuscmb/pranav/codes/persistent_homology/boundary_matrix/bin/boundary_matrix_phat'
indir = '/Xnfs/arthuscmb/pranav/data/CMB/planck2020/SEVEM/SEVEM_NPIPE_sims/TQU/temp/CMB+Noise/'  # with final '/' directory of input Cmb+noise files
outdir = '/Xnfs/arthuscmb/pranav/data/CMB/planck2020/SEVEM/SEVEM_NPIPE_sims/TQU/temp/CMB+Noise/iter4+veMaskFullPD_DS/'  # with final '/' output directory


def write_file(filename, filecontent):
    """ write filecontent into filename, obviously """
    verify_or_create_dir(filename)
    try:
        with open(filename, 'w') as f:
            f.write(filecontent)
    except EnvironmentError as err:
        print('cannot write: {}, {}'.format(filename, err))


def verify_or_create_dir(filename):
    """ Verify if path exist (before a file will be created/written)
        create path if it doesn't exist
    """
    if not os.path.exists(os.path.dirname(filename)):
        try:
            os.makedirs(os.path.dirname(filename), exist_ok=True)
        except OSError as err:
            print('Cannot create directory: {}'.format(err))
            sys.exit(1)


for m in mask:
  for d, f in zip(degrade, fwhm):
  #for d in degrade:
    for i in range(BEGIN, END):
    #for i in itere:
        param_file = '\n'.join([
        'nside=' + str(d),
        'infile='+ indir + 'SEVEM_NPIPE_cmb+noise_sim' + str(i).zfill(4) + '.fits', #input simulationfiles, modify to fit
        'maskfile=/Xnfs/arthuscmb/pranav/data/CMB/ffp10/CommonMask/COM_Mask_CMB-common-Mask-Int_2048_R3.00.fits',
        #'outfile=' + outdir + str(m) + '/' + str(f).zfill(3) + 'a_' + str(d).zfill(4) + '/SEVEM_NPIPE_cmb+noise_sim' + str(i).zfill(4) + '.fits',
        'outfile=' + outdir + str(m) + '/degrade_' + str(d) + '/SEVEM_NPIPE_cmb+noise_sim' + str(i).zfill(4) + '.fits', #masked output fits file
        'level=' + str(m),
        'fwhm_map=' + str(f),
        'fwhm_mask=' + str(f),
        'polarisation=false',
        'pessimistic=false',
        'nlmax='+ str(2*d),
        ''])  # EOF
        #print(param_file)  # debug purpose

        #param_file_name = ''.join([outdir,str(m),'/',str(f).zfill(3), 'a_', str(d).zfill(4),'/params', str(i),  '.txt'])
        #write parameter file
        param_file_name = ''.join([outdir,str(m),'/degrade_',str(d),'/params', str(i),  '.txt']) 
        #print(param_file_name)  # debug purpose
        write_file(param_file_name, param_file)

        sge_script = '\n'.join([
        '#!/bin/bash',
        '#$ -S /bin/bash',
        '# use submission environment',
        '#$ -V',
        # ${JOB_NAME}
        '#$ -N j' + str(d) + '_' + str(i),
        # WARNING: One queue line at a time
        #'#$ -q AMD*',
        '#$ -q CLG5218deb192*',
        '#$ -q CLG6242deb384*',
        '#$ -q SLG6142deb384*',
        '#$ -q *E5-26*128*',
        '#$ -q *E5-26*256*',
        '#$ -q Epyc*',   
        '#$ -q h48-E5-2670deb128,h6-E5-2667v4deb128',
        '#$ -q monointeldeb128',
        #'#$ -q monointeldeb24,monointeldeb48',
        #'#$ -q r820deb768',
        #'#$ -q x5570deb48,x5570deb24',
        #
        # WARNING: core count MUST match queue's
        # See PSMN website
        '#$ -pe openmp32 32',
        # SGE outputs will be in ${SGE_O_WORKDIR}/${JOB_NAME}.[e,o,pe,po]${JOB_ID} by default
        #'#$ -e ' + outdir + str(m) + '/' +  str(f).zfill(3) +'a_' + str(d).zfill(4) + '/e_' + str(i),
        '#$ -e ' + outdir + str(m) + '/degrade_' + str(d) + '/e_' + str(i),
        #'#$ -o ' + outdir + str(m) + '/' +  str(f).zfill(3) +'a_' + str(d).zfill(4) + '/o_' + str(i),
        '#$ -o ' + outdir + str(m) + '/degrade_' + str(d) + '/o_' + str(i),
        #'#$ -m e',
        '#$ -M ' + email,
        '# start job from the directory it was submitted',
        'cd ${SGE_O_WORKDIR}',
        # Add $ENV preparation, if needed:
        # 'source ~/.bashrc',
        # 'module load Python/3.6.1',
        # 'cd /another/path/newdata',
        # Finally, your program(s)
        #binary1 + ' ' + param_file_name + ' | ' + binary2 + ' | ' + binary3 + ' > '+ outdir + str(m) +'/' +  str(f).zfill(3) +'a_' + str(d).zfill(4) + '/SEVEM_NPIPE_cmb+noise_sim' + str(i) + '.dgm',
        binary1 + ' ' + param_file_name + ' | ' + binary2 + ' | ' + binary3 + ' > '+ outdir + str(m) +'/degrade_' + str(d) + '/sample_' + str(i) + '.dgm',
        #'sleep 1',
        ''])  # end of submit script

        #print(sge_script)  # debug purpose
        # write script file
        script_file_name = ''.join(['./', 'job_d', str(d), '_', str(m) , '_', str(i), '.sh'])  # full path or './'
        #print(script_file_name)  # debug purpose
        write_file(script_file_name, sge_script)

        #run the script
        toRun = '{} {}'.format('qsub', script_file_name)
        #print(toRun)  # debug purpose
        os.system(toRun)  # uncomment for production
        #time.sleep(2)
