#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
//#include <CGAL/Triangulation_euclidean_traits_3.h>
#include <CGAL/Delaunay_triangulation_3.h>
//#include <CGAL/Regular_triangulation_3.h>
//#include <CGAL/Regular_triangulation_euclidean_traits_3.h>
#include <CGAL/Triangulation_vertex_base_with_info_3.h>
#include <CGAL/Polygon_2.h>
#include <CGAL/convex_hull_3.h>
#include <CGAL/Polyhedron_3.h>
#include <CGAL/point_generators_3.h>
#include <CGAL/convex_hull_3_to_polyhedron_3.h>
#include <CGAL/algorithm.h>
#include <CGAL/circulator.h>

#include <fstream>
#include <algorithm>
#include <list>
#include <cassert>
#include <vector>


typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
typedef K::Point_3 P_3;

typedef CGAL::Triangulation_ds_cell_base_3<> CbDS;
typedef CGAL::Triangulation_cell_base_3<CbDS> Cb;

struct Vertex_info {
  double value;
  long index;
};


typedef CGAL::Triangulation_vertex_base_with_info_3<Vertex_info,K> VbInfo;
typedef CGAL::Triangulation_data_structure_3<VbInfo>    TDS;
typedef CGAL::Delaunay_triangulation_3<K,TDS>  DT;
typedef DT::Cell_circulator Cell_circulator;
typedef CGAL::Container_from_circulator<Cell_circulator> Container;
//typedef Container::iterator                              Iterator;
typedef DT::Cell_iterator							Iterator;
typedef DT::Vertex_iterator							DV_Iterator;
typedef DT::Point   Point;

typedef CGAL::Polyhedron_3<K> 						Polyhedron_3;
typedef Polyhedron_3::Vertex_iterator                   PV_Iterator;
typedef Polyhedron_3::Facet_iterator                   Facet_iterator;
typedef Polyhedron_3::Facet_handle                   Facet_handle;
typedef Polyhedron_3::Vertex_handle                   PVertex_handle;
typedef Polyhedron_3::Edge_iterator                   Edge_iterator;
typedef Polyhedron_3::Halfedge_iterator                   Halfedge_iterator;
typedef Polyhedron_3::Halfedge_around_facet_circulator Halfedge_facet_circulator;
typedef Polyhedron_3::Halfedge_around_vertex_circulator Halfedge_vertex_circulator;

template<typename DT>
struct Sort_vertices_by_value {

  bool operator() (typename DT::Vertex_handle v1,
		   typename DT::Vertex_handle v2) {
    return v1->info().value < v2->info().value;
  }

};

template<typename DT>
struct Sort_vertices_by_index {

  bool operator() (typename DT::Vertex_handle v1,
		   typename DT::Vertex_handle v2) {
    return v1->info().index < v2->info().index;
  }

};

template<typename OutputIterator>
OutputIterator assign_ids(DT& dt, OutputIterator oit) {

  std::vector<DT::Vertex_handle> vhandles;
  for(DT::Finite_vertices_iterator it  = dt.finite_vertices_begin();
      it!=dt.finite_vertices_end();it++) {
    vhandles.push_back(it);
  }
  std::sort(vhandles.begin(),vhandles.end(),Sort_vertices_by_value<DT>());
  for(long i=0;i<vhandles.size();i++) {
    vhandles[i]->info().index=i;
  }
  std::copy(vhandles.begin(),vhandles.end(),oit);
  return oit;

}


int main(int argc, char** argv)
{
	if(argc<2) {
		std::cerr << "Arguments!" << std::endl;
		exit(1);
	}
	std::vector<Point> points;
	std::vector<Point>::iterator p_iter;
	std::vector<double> data;
	
	std::cerr << "Reading the input.." << std::flush;
	
	std::ifstream ifstr(argv[1]);
	long no_points;
	ifstr >> no_points;
	double trash;
	double a,b,c;
  
	for(long i=0;i<no_points;i++) {
		ifstr >> a >> b >> c >> trash;
        points.push_back(Point(a,b,c));
		data.push_back(trash);
	}
	
	std::vector<double>::iterator d_iter=data.begin(); // set the data iterator to first entry in data column
	
	std::cerr << "done\nComputing triangulation.." << std::flush;
	
	DT dt;
	
	dt.insert(points.begin(),points.end());
	std::cerr << "done\nassigning values to vertices.." << std::flush;
	DT::Vertex_handle vref;
	
	for (p_iter=points.begin(); p_iter!=points.end(); p_iter++) //start with te first entry in the point vector
	{
		vref=dt.nearest_vertex(*p_iter); //find the corresponding vertex in the DT
		vref->info().value = *d_iter++; //assign the data value and increment. this is all done so that to match exact data at exact vertex as given in the data file
	}
	
    std::cerr << "done\nSort the vertices.." << std::flush;

    std::vector<DT::Vertex_handle> vhandles_sorted;

    assign_ids(dt,std::back_inserter(vhandles_sorted));
	
	std::cerr << "done\nExtracting convex hull polyhedron from  triangulation.." << std::endl;
	
	Polyhedron_3 P;
	CGAL::convex_hull_3_to_polyhedron_3(dt,P);
	
	
	std::cerr << "done\nCreate the output...VTU file" << std::endl;
    
	std::size_t count = 1;
	
    std::string fn(argv[1]);
    std::ofstream ofs(fn.replace(fn.find("txt"),3,"vtu").c_str());
	
	ofs<<"# vtk DataFile Version 2.0\nunstructured grid example\nASCII\nDATASET UNSTRUCTURED_GRID\nPOINTS "<<P.size_of_vertices()<<" float\n";
	
	for (  PV_Iterator i = P.vertices_begin(); i != P.vertices_end(); ++i)
	{
		ofs<<i->point()<<"\n";
	}
	ofs<<"\nCELLS "<<P.size_of_halfedges()/2+P.size_of_facets()<<" "<<3*P.size_of_halfedges()/2+4*P.size_of_facets()<<"\n";
	
	for (  Edge_iterator i = P.edges_begin(); i != P.edges_end(); ++i) 
	{
			ofs<< "2" << ' '<<std::distance(P.vertices_begin(), i->vertex())<<" "<<std::distance(P.vertices_begin(), i->opposite()->vertex())<<std::endl;
	}
	
    for (  Facet_iterator i = P.facets_begin(); i != P.facets_end(); ++i) {
        Halfedge_facet_circulator j = i->facet_begin();
        // Facets in polyhedral surfaces are at least triangles.
        CGAL_assertion( CGAL::circulator_size(j) >= 3);
        ofs << CGAL::circulator_size(j) << ' ';
        do {
            ofs << ' ' << std::distance(P.vertices_begin(), j->vertex());
        } while ( ++j != i->facet_begin());
        ofs << std::endl;
    }  
	
	ofs<<"\nCELL_TYPES "<<P.size_of_halfedges()/2+P.size_of_facets()<<std::endl;
	
	for (  Edge_iterator i = P.edges_begin(); i != P.edges_end(); ++i) 
	{
			ofs<< "3"<<std::endl;
	}
	
	for (  Facet_iterator i = P.facets_begin(); i != P.facets_end(); ++i) 
	{
			ofs<< "5"<<std::endl;
	}
	
	ofs<<"\nPOINT_DATA "<<P.size_of_vertices()<<"\nSCALARS temperature float 1\nLOOKUP_TABLE default\n";
	
	for ( PV_Iterator i = P.vertices_begin(); i != P.vertices_end(); i++)
	{
		DT::Vertex_handle v1 = dt.nearest_vertex(i->point());
		if(v1->info().value > -21) ofs<<v1->info().value<<"\n";
		else ofs<<"-21"<<"\n";
	}
	
	/*for(std::vector<DT::Vertex_handle>::iterator it = vhandles_sorted.begin(); it!=vhandles_sorted.end();it++)
	{
		DT::Vertex_handle v = *it;
		ofs<<v->point()<<"\n";   // write out the points to the off file
	}
	
	for(std::vector<DT::Vertex_handle>::iterator it = vhandles_sorted.begin(); it!=vhandles_sorted.end();it++)
	{
		DT::Vertex_handle v = *it;
	    std::cout << "v " << v->info().value << std::endl;
	    std::cout << "s " << v->info().index << std::endl; //start with the smallest vertex value in the sorted DT vetex handles and write the value and index
		
		PV_Iterator i;
		for ( i = P.vertices_begin(); i != P.vertices_end(); i++)
		{
			if(i->point() == v->point())
			{
				break; //find the corresponding vertex in the convex hull, and exit loop when found. 
			}
		}
		
		Halfedge_vertex_circulator j = i->vertex_begin(); // define a circulato of halfedges for this vertex
		std::size_t n = circulator_size(j);
		//std::cerr<<"Number of incident half edges = "<<n<<std::endl;
		std::size_t k = 1;
		
		do //circulate around the half edges incident to the vertex
		{
			DT::Vertex_handle v1 = dt.nearest_vertex(j->opposite()->vertex()->point()); //find the vertex that is pointed to by the opposite half edge
			if(v1->info().value <= v->info().value) // if the value of this vertex is less than the value of the vertex of incidence
			{
				std::cout << "s " << v->info().index <<" "<< v1->info().index<<std::endl; // write the 2-simplex corresonding to this edge
				ofs<<"2 " << v->info().index <<" "<< v1->info().index<<std::endl; // write to off file for visulization
			}
		} while ( ++j != i->vertex_begin());
		
		Halfedge_vertex_circulator p = i->vertex_begin(); // for the circulator of half edges 
		do //do for all the half edges in the circulator
		{
			Facet_handle f = p->facet(); //define the facet handle pointed out to by the half edge
			DT::Vertex_handle v_ref = dt.nearest_vertex(p->vertex()->point()), v_comp[3]; //find the vertex of the DT pointed to by the vertex on which the half edge is incident. this is the reference vertex. also define a an array of vertex handles with size three, to store all the vertices of the current facet
		//std::vector<DT::Vertex_handle> vhandles_comparison[3];
			std::size_t count_facet_vertex=0;
			Halfedge_facet_circulator hfc = f->facet_begin(); //define a halfedge circulator corresponding to the given facet
			int n1 = circulator_size(hfc);
			//std::cerr<<"Size of circulator is "<< n1<<std::endl;
			do //do for all the halfedges
			{
				v_comp[count_facet_vertex] = dt.nearest_vertex(hfc->vertex()->point()); //find the vertices on the DT that corrspond to the vertices of the facet.
				count_facet_vertex++;
			} while ( ++hfc != f->facet_begin());
			if(v_comp[0]->info().value<=v_ref->info().value && v_comp[1]->info().value<=v_ref->info().value && v_comp[2]->info().value<=v_ref->info().value)
				//if all the vertices have values less than or equal to the reference vertex
			{
				std::cout << "s " << v_comp[0]->info().index <<" "<< v_comp[1]->info().index<<" "<<v_comp[2]->info().index<<std::endl; //output the given 2-simplex to the filtration
				ofs<<"3 " << v_comp[0]->info().index <<" "<< v_comp[1]->info().index<<" "<<v_comp[2]->info().index<<std::endl; //write to visualization file
			}
		} while ( ++p != i->vertex_begin());
	}
	
	std::cout<<"!"<<std::endl;*/
	return 0;
	
}
