/*
 *  This file is part of TopoS2.
 *
 *  TopoS2 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  TopoS2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with ToposS2; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *
 *  Copyright (C) Pratyush Pranav
 *  \author Pratysh Pranav <pratyuze@gmail.com>, <pratyush.pranav@ens-lyon.fr>
 */





#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
//#include <CGAL/Triangulation_euclidean_traits_3.h>
#include <CGAL/Delaunay_triangulation_3.h>
//#include <CGAL/Regular_triangulation_3.h>
//#include <CGAL/Regular_triangulation_euclidean_traits_3.h>
#include <CGAL/Triangulation_vertex_base_with_info_3.h>
#include <CGAL/Polygon_2.h>
//#include <CGAL/convex_hull_3.h>
#include <CGAL/Polyhedron_3.h>
#include <CGAL/point_generators_3.h>
#include <CGAL/convex_hull_3_to_polyhedron_3.h>
#include <CGAL/algorithm.h>
#include <CGAL/circulator.h>

#include <fstream>
#include <algorithm>
#include <list>
#include <cassert>
#include <vector>
#include <queue>
#include <map>
#include <iostream>
#include <set>
#include <sstream>
#include <limits>



template <class Refs, class Traits>
struct My_vertex :
 public CGAL::HalfedgeDS_vertex_base<Refs, CGAL::Tag_true, typename Traits::Point_3> {
   typedef typename Traits::Point_3 Point;
   typedef CGAL::HalfedgeDS_vertex_base<Refs, CGAL::Tag_true, Point> Base;
   double value;
   long index;
   My_vertex() : Base(){}
   My_vertex(const Point& p) : Base(p) {}
};


// An items type using my face.
struct My_items : public CGAL::Polyhedron_items_3 {
   template <class Refs, class Traits>
   struct Vertex_wrapper {
       typedef My_vertex<Refs,Traits> Vertex;
   };
};


typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
typedef CGAL::Triangulation_ds_cell_base_3<> CbDS;
typedef CGAL::Triangulation_cell_base_3<CbDS> Cb;

struct Vertex_info {
  double value;
  long index;
};


typedef CGAL::Triangulation_vertex_base_with_info_3<Vertex_info,K> VbInfo;
typedef CGAL::Triangulation_data_structure_3<VbInfo>    TDS;
typedef CGAL::Delaunay_triangulation_3<K,TDS>  DT;
typedef DT::Cell_circulator Cell_circulator;
typedef DT::Cell_iterator							Iterator;
typedef DT::Vertex_iterator							DV_Iterator;
typedef DT::Point   Point;


typedef CGAL::Polyhedron_3<K,My_items> 						Polyhedron_3;
typedef Polyhedron_3::Vertex_handle                   PVertex_handle;
typedef Polyhedron_3::Facet_handle                   Facet_handle;
typedef Polyhedron_3::Vertex_iterator                   PV_Iterator;
typedef Polyhedron_3::Facet_iterator                   Facet_iterator;
typedef Polyhedron_3::Edge_iterator                   Edge_iterator;
typedef Polyhedron_3::Halfedge_iterator                   Halfedge_iterator;
typedef Polyhedron_3::Halfedge_around_facet_circulator Halfedge_facet_circulator;
typedef Polyhedron_3::Halfedge_around_vertex_circulator Halfedge_vertex_circulator;

template<typename Polyhedron_3>
struct Sort_vertices_by_value_P {

  bool operator() (typename Polyhedron_3::Vertex_handle v1,
		   typename Polyhedron_3::Vertex_handle v2) {
    return v1->value > v2->value;
  }

};

template<typename Polyhedron_3>
struct Sort_vertices_by_index_P {

  bool operator() (typename Polyhedron_3::Vertex_handle v1,
		   typename Polyhedron_3::Vertex_handle v2) {
    return v1->index < v2->index;
  }

};

template<typename DT>
struct Sort_vertices_by_value {

  bool operator() (typename DT::Vertex_handle v1,
		   typename DT::Vertex_handle v2) {
    return v1->info().value > v2->info().value;
  }

};

template<typename DT>
struct Sort_vertices_by_index {

  bool operator() (typename DT::Vertex_handle v1,
		   typename DT::Vertex_handle v2) {
    return v1->info().index < v2->info().index;
  }

};

template<typename OutputIterator>
OutputIterator assign_ids_p(Polyhedron_3& P, OutputIterator oit) {

  std::vector<Polyhedron_3::Vertex_handle> vhandles;
  for(Polyhedron_3::Vertex_iterator it  = P.vertices_begin();
      it!=P.vertices_end();it++) {
    vhandles.push_back(it);
  }
  std::sort(vhandles.begin(),vhandles.end(),Sort_vertices_by_value_P<Polyhedron_3>());
  for(long i=0;i<vhandles.size();i++) {
    vhandles[i]->index=i;
  }
  std::copy(vhandles.begin(),vhandles.end(),oit);
  return oit;

}

template<typename OutputIterator>
OutputIterator assign_ids(DT& dt, OutputIterator oit) {

  std::vector<DT::Vertex_handle> vhandles;
  for(DT::Finite_vertices_iterator it  = dt.finite_vertices_begin();
      it!=dt.finite_vertices_end();it++) {
    vhandles.push_back(it);
  }
  std::sort(vhandles.begin(),vhandles.end(),Sort_vertices_by_value<DT>());
  for(long i=0;i<vhandles.size();i++) {
    vhandles[i]->info().index=i;
  }
  std::copy(vhandles.begin(),vhandles.end(),oit);
  return oit;

}


int main(int argc, char** argv)
{
	if(argc<1) {
		std::cerr << "Arguments!" << std::endl;
		exit(1);
	}
	std::vector<Point> points;
	std::vector<Point>::iterator p_iter;
	std::vector<double> data;
	
	std::cerr << "Reading the input.." << std::flush;
	
	//std::ifstream ifstr(argv[1]);
	//long no_points;
	//ifstr >> no_points;
	double trash;
	double a,b,c;
	//char curr[4096];
	//std::cin.getline(curr,4096);
	//std::cin.getline(curr,4096);
	//std::cin.getline(curr,4096);
	//std::cin.getline(curr,4096);
	//std::cin.getline(curr,4096);
	//std::cin.getline(curr,4096);
	//std::cin.getline(curr,4096);
	//std::cin.getline(curr,4096);

	while(std::cin) {
	  char curr[4096];
	  std::cin.getline(curr,4096);
	  std::string str(curr);
	  //std::cerr << str << std::endl;
	  if(str=="") {
            continue;
	  }
	  std::stringstream sstr(str);
	  sstr>>a >> b >> c >> trash;
	  points.push_back(Point(a,b,c));
	  data.push_back(trash);
	}

	//std::cerr << "Reading the input.." << std::flush;
	//<no_points;i++) {
	//	ifstr >> a >> b >> c >> trash;
        //points.push_back(Point(a,b,c));
	//data.push_back(trash);
	//}
	
	std::vector<double>::iterator d_iter=data.begin(); // set the data iterator to first entry in data column
	
	std::cerr << "done\nComputing triangulation.." << std::flush;
	
	DT dt;
	
	dt.insert(points.begin(),points.end());
	std::cerr << "done\nassigning values to vertices.." << std::flush;
	DT::Vertex_handle vref;
	
	for (p_iter=points.begin(); p_iter!=points.end(); p_iter++) //start with te first entry in the point vector
	{
		vref=dt.nearest_vertex(*p_iter); //find the corresponding vertex in the DT
		vref->info().value = *d_iter++; //assign the data value and increment. this is all done so that to match exact data at exact vertex as given in the data file
	}
	
	std::cerr << "done\nExtracting convex hull polyhedron from  triangulation.." << std::flush;
	
	Polyhedron_3 P;
	CGAL::convex_hull_3_to_polyhedron_3(dt,P);
	
	std::cerr << "done\nAssigning values to the vertices of the polyhedron.." << std::flush;
	
	for ( PV_Iterator i = P.vertices_begin(); i != P.vertices_end(); i++)
	{
		vref=dt.nearest_vertex(i->point());
		i->value = vref->info().value;
	}
	
    std::cerr << "done\nSort the vertices of the polyhedron.." << std::flush;

    //std::vector<DT::Vertex_handle> vhandles_sorted;
	
	std::vector<Polyhedron_3::Vertex_handle> vhandles_sorted;

    assign_ids_p(P,std::back_inserter(vhandles_sorted));
	
	
	std::cerr << "done\nCreate the output..." << std::endl;
    
	
	for(std::vector<Polyhedron_3::Vertex_handle>::iterator it = vhandles_sorted.begin(); it!=vhandles_sorted.end();it++)
	{
		Polyhedron_3::Vertex_handle v = *it;
	    std::cout << "v " << v->value << std::endl;
	    std::cout << "s " << v->index << std::endl; //start with the largest vertex value in the sorted DT vetex handles and write the value and index
		
		Halfedge_vertex_circulator j = v->vertex_begin(); // define a circulato of halfedges for this vertex
		std::size_t n = circulator_size(j);
		//std::cerr<<"Number of incident half edges = "<<n<<std::endl;
		
		do //circulate around the half edges incident to the vertex
		{
			Polyhedron_3::Vertex_handle v1 = j->opposite()->vertex(); //find the vertex that is pointed to by the opposite half edge
			if(v1->value >= v->value && v1->index < v->index) // if the value of this vertex is more than the value of the vertex of incidence
			{
				std::cout << "s " << v1->index <<" "<< v->index<<std::endl; // write the 2-simplex corresonding to this edge
			}
		} while ( ++j != v->vertex_begin());
		
		Halfedge_vertex_circulator p = v->vertex_begin(); // for the circulator of half edges 
		do //do for all the half edges in the circulator
		{
			Facet_handle f = p->facet(); //define the facet handle pointed out to by the half edge
			Polyhedron_3::Vertex_handle v_ref = p->vertex(), v_comp[3]; 
		//std::vector<DT::Vertex_handle> vhandles_comparison[3];
			std::size_t count_facet_vertex=0;
			Halfedge_facet_circulator hfc = f->facet_begin(); //define a halfedge circulator corresponding to the given facet
			int n1 = circulator_size(hfc);
			//std::cerr<<"Size of circulator is "<< n1<<std::endl;
			std::vector<Polyhedron_3::Vertex_handle> verts;
			do //do for all the halfedges
			{
				v_comp[count_facet_vertex] = hfc->vertex(); //find the vertices on the DT that corrspond to the vertices of the facet.
				verts.push_back(hfc->vertex());
				count_facet_vertex++;
			} while ( ++hfc != f->facet_begin());
			std::sort(verts.begin(),verts.end(),Sort_vertices_by_index_P<Polyhedron_3>());
	        if(verts[2]->index==v->index) 
			{
	  			std::cout << "s "<< verts[0]->index << " "<< verts[1]->index << " "<< verts[2]->index << std::endl;
	        }
			/*if(v_comp[0]->info().value<=v_ref->info().value && v_comp[1]->info().value<=v_ref->info().value && v_comp[2]->info().value<=v_ref->info().value)
				//if all the vertices have values less than or equal to the reference vertex
			{
				std::cout << "s " << v_comp[0]->info().index <<" "<< v_comp[1]->info().index<<" "<<v_comp[2]->info().index<<std::endl; //output the given 2-simplex to the filtration
				ofs<<"3 " << v_comp[0]->info().index <<" "<< v_comp[1]->info().index<<" "<<v_comp[2]->info().index<<std::endl; //write to visualization file
			}*/
		} while ( ++p != v->vertex_begin());
	}
	
	std::cout<<"!"<<std::endl;
	return 0;
	
}
