#ifndef CUBE_VERTEX_H
#define CUBE_VERTEX_H 1

#include "octree.h"

template<int D>
struct Cube_vertex {
  
  Ocnode<D> *node; // The vertex is a vertex of this cube
  Index_type index; // which one in lex order

  long coords[D];

  Ocnode<D> *adj_terminals[Ocnode<D>::no_children];

  Cube_vertex() : node(NULL), index(-1) {}

  Cube_vertex(Ocnode<D> *node,Index_type index) : node(node),index(index) {
    if(node) {
      for(int i=0;i<D;i++) {
	if(index & (1 << i)) {
	  coords[i] = node->coordinates[2*i+1];
	} else {
	  coords[i] = node->coordinates[2*i];
	}
      }
    }
  }

  bool operator== (const Cube_vertex<D>& a) {
    for(int i=0;i<D;i++) {
      if(this->coords[i]!=a.coords[i]) {
	return false;
      }
    }
    return true;
  }

};

template<int D>
struct Cube_vertex_compare {

  bool operator() (const Cube_vertex<D>& a, const Cube_vertex<D>& b) {
    for(int i=0;i<D;i++) {
      if(a.coords[i]<b.coords[i]) {
	return true;
      }
      if(a.coords[i]>b.coords[i]) {
	return false;
      }
    }
    return false;
  }

};


template<int D>
Ocnode<D> *find_adjacent_terminal(Cube_vertex<D> v) {
  Ocnode<D> *node=v.node;
  if(! node) {
    return NULL;
  }
  //std::cerr << "XXX "  << node << " " << v.index << std::endl;
  Index_type index=v.index;
  if(node->is_terminal) {
    return node;
  }
  if(! node->is_interior_after_prune) {
    Ocnode<D> *curr=node;
    while(!curr->is_terminal) { 
      curr=curr->parent;
    }
    assert(curr->is_terminal);
    return curr;
  }
  // remaining case: interior cube
  Ocnode<D> *curr=node;
  while(curr && !curr->is_terminal) {
    curr=curr->child[index];
    //assert(curr);
  }
  //assert(curr->is_terminal);
  return curr;
}

template<int D>
void find_all_adjacent_terminals(Cube_vertex<D>& v) {
  Ocnode<D> *node = v.node;
  Index_type index = v.index;
  assert(node);
  Ocnode<D> *curr_neighbor=NULL;
  
  int flip_pattern=0;
  for(int i=0;i<D;i++) {
    flip_pattern = flip_pattern | (1<<i);
  }

  for(Index_type i=0;i<Ocnode<D>::no_children;i++) {
    curr_neighbor=node;
    Index_type dir_tt = (index ^ i);
    for(int j=0;j<D;j++) {
      if(! (dir_tt & (1 << j))) {
	if(index & (1 << j)) {
	  curr_neighbor = curr_neighbor->neighbor[2*j+1];
	} else {
	  curr_neighbor = curr_neighbor->neighbor[2*j];
	}
	if(! curr_neighbor) {
	  break;
	}
      }
    }
    v.adj_terminals[i]=find_adjacent_terminal
      (Cube_vertex<D>(curr_neighbor,i^flip_pattern));
  }
}

#endif
