#ifndef PRUNE_H_
#define PRUNE_H_ 1

#include "flags.h"

template<int D>
void random_prune(Ocnode<D>* node,int w) {
  
  node->is_terminal=true;

  if(node->is_leaf) {
    return;
  }
  
  int r = rand()%100;
  if(r<w) {
    node->is_terminal=false;
    node->is_interior_after_prune=true;
    for(long i=0;i<Ocnode<D>::no_children;i++) {
      if(node->child[i]) {
	random_prune(node->child[i],w);
      }
    }
  } else {
    return;
  }
}

template<int D>
void directed_prune_(Ocnode<D>* node,std::ifstream& ifstr) {
  
  int c;
  ifstr >> c;

  if(!c) {

    node->is_terminal=true;

  } else {
    assert(! node->is_leaf);
    node->is_interior_after_prune=true;
    for(long i=0;i<Ocnode<D>::no_children;i++) {
      if(node->child[i]) {
	directed_prune_<D>(node->child[i],ifstr);
      }
    }
  }
  return;
}

template<int D> void directed_prune(Ocnode<D>* node, std::string filename) {
  std::ifstream ifstr(filename.c_str());
  if(!ifstr) {
    std::cerr << "File " << filename << " not found" << std::endl;
    std::exit(-1);
  }
  directed_prune_<D>(node,ifstr);
}

template<int D>
struct Compare_variations {

  bool operator() (Ocnode<D>* a, Ocnode<D>* b) {
    return a->variation < b->variation;
  }

};

template<int D>
bool is_in(Ocnode<D>* node, long* coords) {
  assert(node!=NULL);
  assert(coords!=NULL);
  for(int i=0;i<D;i++) {
    if( (node->coordinates[2*i] > coords[i]) ||
	(node->coordinates[2*i+1] < coords[i]) ) {
      return false;
    }
  }
  return true;  
}


template<int D>
Ocnode<D>* find_terminal_containing(Ocnode<D>* node, long* coords) {
  if(node->is_terminal) {
    return node;
  } else {
    for(long i=0;i<Ocnode<D>::no_children;i++) {
      if(node->child[i]) {
	if(is_in(node->child[i],coords)) {
	  return find_terminal_containing(node->child[i],coords);
	}
      }
    }
  }
  return 0; // should never happen
}



// REMARK: bounds and max_val are only needed for pgm files!!
template<int D>
std::pair<float,long>
#if PRUNE_TO_MAXIMAL_TERMINALS
  prune(Ocnode<D>* node, int* bounds, float max_val,long max_terminals) {
#else
  prune(Ocnode<D>* node, int* bounds, float max_val, float threshold) {
#endif
  
  std::pair<float,long> result;

  //std::cerr << "Threshold is " << threshold << std::endl;

#if PRUNE_TO_MAXIMAL_TERMINALS
  if(max_terminals==0) {
    max_terminals=std::numeric_limits<long>::max();
  }
#endif

  std::priority_queue<Ocnode<D>*,
    std::vector<Ocnode<D>*>,
    Compare_variations<D> > pq;

  node->is_terminal=true;
  long no_terminals=1;
#if WRITE_PGM
  long run=0;
#endif
#if USES_COMPACTIFIED_3_SPHERE
  no_terminals++;
#endif
  pq.push(node);
  float fitness=pq.top()->variation;
  while(!pq.empty() &&
#if PRUNE_TO_MAXIMAL_TERMINALS
	no_terminals<max_terminals
#else
	fitness>threshold
#endif
	) {

#if WRITE_PGM
    if(run%PGM_DIST==0) {
      int suffix=run/PGM_DIST;
      FILE* fileh;
      std::stringstream sstream;
      sstream << "imgs/bleh_" << suffix << ".pgm";
      std::cout << "Write " << sstream.str() << std::endl;
      fileh=fopen(sstream.str().c_str(),"w");
      std::cerr << "fitness=" << fitness << std::endl;
      long b_count=0;
      long X=bounds[0]*bounds[1];
      Converter<D> cv(bounds);
      fprintf(fileh,"P2\n%d %d\n%d\n",bounds[0],bounds[1],(long)max_val);
      for(int i=0;i<X;i++) {
	long coords[] = {cv.get_coord(i,0),cv.get_coord(i,1)};
	fprintf(fileh,"%d ",(long)(find_terminal_containing(node,coords)->value));
	b_count++;
	if(b_count==bounds[1]) {
	  fprintf(fileh,"\n");
	  b_count=0;
	}
      }
    }
    run++;
#endif
    Ocnode<D>* curr=pq.top();
    assert(!curr->is_leaf);
    assert(curr->is_terminal);
#if PRUNE_TO_MAXIMAL_TERMINALS
    if(max_terminals-no_terminals<7) {
      // Count children
      int no_children=0;
      for(int i=0;i<Ocnode<D>::no_children;i++) {
	if(curr->child[i]) {
	  no_children++;
	}
      }
      if(max_terminals-(no_terminals+no_children-1)<0) {
	break;
      }
    }    
#endif
    pq.pop();
    curr->is_terminal=false;
    no_terminals--;
    curr->is_interior_after_prune=true;
    for(int i=0;i<Ocnode<D>::no_children;i++) {
      if(curr->child[i]) {
	no_terminals++;
	curr->child[i]->is_terminal=true;
	if(! curr->child[i]->is_leaf) {
	  pq.push(curr->child[i]);
	}
      }
    }
    fitness=pq.top()->variation;
  }
  fitness = pq.empty() ? 0. : pq.top()->variation;
  return std::make_pair(fitness,no_terminals);
}


#endif
